#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/wait.h>

#define FILE_BLOCK	64
#define TEA_BLOCK	8

void TEA(char block[8], char password[16], int function)
{
 uint32_t *text = (uint32_t*)block;
 uint32_t *key = (uint32_t*)password;
 uint32_t v0=text[0], v1=text[1];
 uint32_t delta=0x9e3779b9;
 uint32_t k0=key[0], k1=key[1], k2=key[2], k3=key[3];
 uint32_t sum;
 unsigned short i;

 if (function==0) // encrypt
   sum=0;
 else
     sum=0xC6EF3720;
 for (i=0; i<32; i++)
    if (function==0) // encrypt
      {
       sum += delta;
       v0 += ((v1<<4)+k0) ^ (v1+sum) ^ ((v1>>5)+k1);
       v1 += ((v0<<4)+k2) ^ (v0+sum) ^ ((v0>>5)+k3);
      }
    else
        {
         v1 -= ((v0<<4)+k2) ^ (v0+sum) ^ ((v0>>5)+k3);
         v0 -= ((v1<<4)+k0) ^ (v1+sum) ^ ((v1>>5)+k1);
         sum -= delta;
        }
 text[0]=v0; text[1]=v1;
}


int main (int argc, char* argv[])
{
	int file, n, pid, status;
	char file_buffer[FILE_BLOCK];
	char block[TEA_BLOCK];
    char password[16];
	int ppf[2], pfn[2];
	
	if (argc != 4)
	{
		printf("Uso: %s file-input file-output parola-chiave\r\n", argv[0]);
		return -1;
	}
	if (strlen(argv[3]) > 16)
	{
		printf("Parola chiave errata\r\n");
		return -1;
	}
    memset(password, 0, 16);
    memcpy(password, argv[3], strlen(argv[3]));
	// creazione pipe
	if (pipe(ppf) < 0)
	{
		printf("Errore creazione pipe\r\n");
		return -1;
	}
	// fork processo figlio
	pid = fork();
	if (pid < 0)
	{
		printf("Errore invocazione fork\r\n");
		// chiusura estremita’ pipe
		close(ppf[0]);
		close(ppf[1]);
		return -1;
	}
	if (pid > 0)
	{
		// processo padre
		close(ppf[0]); // chiusura uscita pipe
		// apertura in lettura file
		file = open(argv[1], O_RDONLY);
		if (file <= 0)
		{
			printf("Errore apertura file \"%s\"\r\n",argv[1]);
			close(ppf[1]); // chiusura entrata pipe
			wait(&status); // attesa terminazione processo figlio
			return -1;
		}
		// lettura da file di blocchi di dati di 64 byte e inserimento nella pipe
		while ((n = read(file, file_buffer, FILE_BLOCK)) > 0)
			write(ppf[1], file_buffer, n);
		close(file); // chiusura file
		close(ppf[1]); // chiusura entrata pipe
		wait(&status); // attesa terminazione processo figlio
		return 0; // terminazione processo padre
	}
	else
	{
			//processo figlio
			close(ppf[1]);
            // creazione pipe
			if (pipe(pfn) < 0)
			{
                printf("Errore creazione pipe\r\n");
                return -1;
			}
            // fork processo nipote
			pid = fork();
			if (pid < 0)
			{
				printf("Errore invocazione fork\r\n");
				// chiusura estremita’ pipe
				close(pfn[0]);
				close(pfn[1]);
				return -1;
			}
			if (pid > 0)
			{
                // processo figlio
                close(pfn[0]);
                // lettura da pipe di blocchi di dati di 8 byte, cifratura e inserimento nella pipe
                while ((n = read(ppf[0], block, TEA_BLOCK)) > 0)
                {
#ifdef ENCRYPT
                   TEA(block, password, 0);
#endif
#ifdef DECRYPT
			       TEA(block, password, 1);
#endif
			       write(pfn[1], block, TEA_BLOCK);
                }
			    close(ppf[0]); // chiusura uscita pipe
			    close(pfn[1]); // chiusura entrata pipe
			    wait(&status); // attesa terminazione processo nipote
			    return 0; // terminazione processo figlio
			}
			else
				{
				 // processo nipote
				 close(pfn[1]);
				 file = open(argv[2], O_CREAT|O_WRONLY, 0666);
				 if (file <= 0)
                 {
                    printf("Errore apertura file \"%s\"\r\n",argv[2]);
			   		close(pfn[0]); // chiusura entrata pipe
			   		return -1;
		      	 }
				 while ((n = read(pfn[0], file_buffer, FILE_BLOCK)) > 0)
					write(file, file_buffer, n);
				 close(pfn[0]);
				 close(file);
				 return 0;
				}
		}
}
