
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "shared.h"
#include "semaphore.h"

#define TRUE 1
#define FALSE 0

#define KEY_FILE "key_file"

// struttura di definizione dell'area di memoria condivisa
struct SHARED
{
 unsigned int number; // numero sequenziale
 int request_semaphore; // semaforo per segnalazione richiesta dato
 int data_semaphore; // semaforo per segnalazione dato disponibile
};

struct SHARED *shared; // puntatore area di memoria condivisa


int main(int argc, char* argv[])
{
 int key; // chiave per creazione area di memoria condivisa
 
  // generazione chiave per allocazione area di memoria condivisa
 key = ftok(KEY_FILE, '@');
 if (key == -1)
   {
    printf("ftok error\r\n");
    return 0;
   }
 // allocazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, TRUE);
 if (shared == (struct SHARED*)(-1))
   {
    printf("shared_memory_alloc error\r\n");
    return 0;
   }
 // inizializzazione semafori
 shared->request_semaphore = semaphore_init(1);
 if (shared->request_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    shared_memory_free(shared);
    return 0;
   }
 shared->data_semaphore = semaphore_init(0);
 if (shared->data_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    semaphore_remove(shared->request_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 // inizializzazione valore numerico
 shared->number = 0;
 // esecuzione ciclo processo produttore
 do
   {
    semaphore_wait(shared->request_semaphore);
    shared->number++;
    semaphore_signal(shared->data_semaphore);
   } while (TRUE);
 semaphore_remove(shared->request_semaphore);
 semaphore_remove(shared->data_semaphore);
 shared_memory_free(shared); // rimozione area memoria condivisa
 return 1;
}
