#include <stdio.h>
#include <string.h>
#include <stdint.h>
#include <stdlib.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/wait.h>

#define PROGRAM "ACQUIRE"

uint32_t CRC32(char byte[], int n)
{
 static uint32_t table[256];
 static int table_computed = 0;
 uint32_t rem;
 uint8_t octet;
 int i, j;
 char *p, *q;
 unsigned long CRC = 0;

 if (table_computed == 0)
   {
    for (i = 0; i < 256; i++)
       {
        rem = i;
        for (j = 0; j < 8; j++)
           {
            if (rem & 1)
              {
               rem >>= 1;
               rem ^= 0xedb88320;
              }
            else
                rem >>= 1;
           }
        table[i] = rem;
       }
    table_computed = 1;
   }
 
 CRC = ~CRC;
 q = byte + n;
 for (p = byte; p < q; p++)
    {
     octet = *p;
     CRC = (CRC >> 8) ^ table[(CRC & 0xff) ^ octet];
    }
 return ~CRC;
}


int spawn(const char program[], char* arguments[])
{
 int pid;
 // clonazione del processo
 pid = fork ();
 if (pid < 0)
   return -1; // errore clonazione processo
 if (pid > 0)
   return 0; // codice di successo restituito al processo padre
 execv(program, arguments);
 abort(); // errore esecuzione programma
}




int main(int argc, char* argv[])
{
 int fifo, n, pid, status;
 char buffer[1024];
 int p[2];
 FILE* file;
 uint32_t crc32;
 char* argument[2]; // argomento per invocazione programma
 
    if (argc != 3)
	{
		printf("Uso: %s file-output numero-pacchetti\r\n", argv[0]);
		return -1;
	}
	// predisposizione argomenti invocazione programma
    argument[0] = (char*)malloc(strlen(PROGRAM)+1);
    argument[1] = (char*)malloc(strlen(argv[2])+1);
    argument[2] = NULL;
    strcpy(argument[0], PROGRAM); // nome del programma
    strcpy(argument[1], argv[2]); // argomento del programma
	
	// esecuzione programma come processo separato
    if (spawn(PROGRAM, argument) < 0)
    {
        printf("Errore invocazione spawn\r\n");
        return -1;
   }
	// creazione pipe
	if (pipe(p) < 0)
	{
		printf("Errore creazione pipe\r\n");
		return -1;
	}
	// fork processo figlio
	pid = fork();
	if (pid < 0)
	{
		printf("Errore invocazione fork\r\n");
		// chiusura estremita’ pipe
		close(p[0]);
		close(p[1]);
		return -1;
	}
	if (pid > 0)
	{
		// processo padre
		close(p[0]); // chiusura uscita pipe
		// apertura in lettura FIFO
		fifo = open("INPUT", O_RDONLY);
		if (fifo <= 0)
		{
			printf("Errore apertura FIFO\r\n");
			close(p[1]); // chiusura entrata pipe
			wait(&status); // attesa terminazione processo figlio
			return -1;
		}
		// lettura dalla FIFO di pacchetti di dati di 1024 byte e inserimento nella pipe
		while ((n = read(fifo, buffer, 1024)) > 0)
			write(p[1], buffer, n);
		close(fifo); // chiusura file
		close(p[1]); // chiusura entrata pipe
		wait(&status); // attesa terminazione processo figlio
		return 0; // terminazione processo padre
	}
	else
	{
			//processo figlio
			close(p[1]); // chiusura entrata pipe
            file = fopen(argv[1], "w");
            if (file == NULL)
            {
                printf("Errore apertura file \"%s\"\r\n", argv[1]);
                close(p[0]); // chiusura entrata pipe
			    return -1;
            }
            // lettura da pipe di pacchetti di dati di 1024 byte, calcolo CRC32 e scrittura nel file
            while ((n = read(p[0], buffer, 1024)) > 0)
                {
                   crc32 =  CRC32(buffer, n);
                   fprintf(file, "%XH\r\n", crc32); 
                }
			    close(p[0]); // chiusura uscita pipe
			    fclose(file);
                return 0; // terminazione processo figlio
	}
}
