
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "shared.h"
#include "semaphore.h"
#include "robot.h"

#define BUFFER_SIZE 8

#define TRUE 1
#define FALSE 0


// struttura di definizione dell'area di memoria condivisa
struct SHARED
{
 struct POSITION buffer[BUFFER_SIZE]; // buffer condiviso
 unsigned int count; // numero di dati nel buffer
 int space_semaphore; // semaforo per segnalazione spazio nel buffer
 int data_semaphore; // semaforo per segnalazione dati nel buffer
 int mutex_semaphore; // semaforo per accesso mutuamente esclusivo
};

FILE *output_file; // identificatore file di output
struct SHARED *shared; // puntatore area di memoria condivisa
int buffer_read_index; // indice per lettura blocchi dal buffer


int main(int argc, char* argv[])
{
 int count;
 int key; // chiave per creazione area di memoria condivisa
 
 if (argc != 2)
   return 0;
 // generazione chiave per allocazione area di memoria condivisa
 key = ftok(argv[0], '@');
 if (key == -1)
   {
    printf("ftok error\r\n");
    return 0;
   }
 // associazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, FALSE);
 if (shared == (struct SHARED*)(-1))
   {
    printf("shared_memory_alloc error\r\n");
    return 0;
   }
 // apertura file di output in scrittura
 output_file = fopen(argv[1], "w");
 if (output_file == NULL)
   {
    printf("file open error\r\n");
    shared_memory_free(shared);
    return 0;
   }
 buffer_read_index = 0; // inizializzazione indice di lettura
 // esecuzione ciclo processo produttore
 do
   {
    semaphore_wait(shared->data_semaphore);
    // verifica effettiva disponibilita' dati
    semaphore_wait(shared->mutex_semaphore);
    count = shared->count;
    semaphore_signal(shared->mutex_semaphore);
    if (count==0)
      break;
    printf("%u,%lu.%03u,%.1f,%.1f,%.1f\r\n", shared->buffer[buffer_read_index].robot, shared->buffer[buffer_read_index].second, shared->buffer[buffer_read_index].millisecond, shared->buffer[buffer_read_index].x, shared->buffer[buffer_read_index].y, shared->buffer[buffer_read_index].z);
    fprintf(output_file, "%u,%lu.%03u,%.1f,%.1f,%.1f\r\n", shared->buffer[buffer_read_index].robot, shared->buffer[buffer_read_index].second, shared->buffer[buffer_read_index].millisecond, shared->buffer[buffer_read_index].x, shared->buffer[buffer_read_index].y, shared->buffer[buffer_read_index].z); 
    fflush(output_file);
    // aggiornamento indice di scrittura nel file
    buffer_read_index = (buffer_read_index+1) % BUFFER_SIZE;
    // aggiornamento numero blocchi dati e copia flag fine dati
    semaphore_wait(shared->mutex_semaphore);
    count = shared->count--;
    semaphore_signal(shared->mutex_semaphore);
    semaphore_signal(shared->space_semaphore);
   } while (TRUE);
 fclose(output_file); // chiusura file di output
 // rimozione semafori e area di memoria condivisa
 semaphore_remove(shared->space_semaphore);
 semaphore_remove(shared->data_semaphore);
 semaphore_remove(shared->mutex_semaphore);
 shared_memory_free(shared);
 return 1;
}
