
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "shared.h"
#include "semaphore.h"
#include "robot.h"


#define CONSUMER_PROGRAM "./consumer"
#define BUFFER_SIZE 8

#define TRUE 1
#define FALSE 0


int spawn(const char program[], char* arguments[])
{
 int pid;
 // clonazione del processo
 pid = fork ();
 if (pid < 0)
   return -1; // errore clonazione processo
 if (pid > 0)
   return 0; // codice di successo restituito al processo padre
 execv(program, arguments);
 abort(); // errore esecuzione programma
}


// struttura di definizione dell'area di memoria condivisa
struct SHARED
{
 struct POSITION buffer[BUFFER_SIZE]; // buffer condiviso
 unsigned int count; // numero di dati nel buffer
 int space_semaphore; // semaforo per segnalazione spazio nel buffer
 int data_semaphore; // semaforo per segnalazione dati nel buffer
 int mutex_semaphore; // semaforo per accesso mutuamente esclusivo
};

struct SHARED *shared; // puntatore area di memoria condivisa
int buffer_write_index; // indice per scrittura blocchi nel buffer


int main(int argc, char* argv[])
{
 int key; // chiave per creazione area di memoria condivisa
 struct POSITION position; // dato temporaneo
 char* argument[2]; // argomento per invocazione programma

 if (argc != 2)
   {
    printf("Uso: %s output-file\r\n",argv[0]);
    return 0;
   }
 // predisposizione argomenti invocazione programma
 argument[0] = (char*)malloc(strlen(CONSUMER_PROGRAM)+1);
 argument[1] = (char*)malloc(strlen(argv[1])+1);
 argument[2] = NULL;
 strcpy(argument[0], CONSUMER_PROGRAM); // nome del programma
 strcpy(argument[1], argv[1]); // argomento del programma
 // generazione chiave per allocazione area di memoria condivisa
 key = ftok(CONSUMER_PROGRAM, '@');
 if (key == -1)
   {
    printf("ftok error\r\n");
    return 0;
   }
 // allocazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, TRUE);
 if (shared == (struct SHARED*)(-1))
   {
    printf("shared_memory_alloc error\r\n");
    return 0;
   }
 // inizializzazione semafori
 shared->space_semaphore = semaphore_init(BUFFER_SIZE);
 if (shared->space_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    shared_memory_free(shared);
    return 0;
   }
 shared->data_semaphore = semaphore_init(0);
 if (shared->data_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    semaphore_remove(shared->space_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 shared->mutex_semaphore = semaphore_init(1);
 if (shared->mutex_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    semaphore_remove(shared->space_semaphore);
    semaphore_remove(shared->data_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 // inizializzazione variabili condivise e indice di scrittura
 buffer_write_index = 0;
 shared->count = 0;
  // esecuzione programma consumatore come processo separato
 if (spawn(CONSUMER_PROGRAM, argument) < 0)
   {
    printf("spawn error\r\n");
    semaphore_remove(shared->space_semaphore);
    semaphore_remove(shared->data_semaphore);
    semaphore_remove(shared->mutex_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 // esecuzione ciclo processo produttore
 do
   {
    // acquisizione dati e copia nel buffer condiviso
    if (getPosition(&position))
    {
     semaphore_wait(shared->space_semaphore);
     shared->buffer[buffer_write_index] = position;
     // aggiornamento indice di scrittura nel buffer
     buffer_write_index = (buffer_write_index+1) % BUFFER_SIZE;
     semaphore_wait(shared->mutex_semaphore);
     shared->count++;
     semaphore_signal(shared->mutex_semaphore);
     semaphore_signal(shared->data_semaphore);
    }
   } while (TRUE);
 // segnalazione per processo consumatore in attesa
 semaphore_signal(shared->data_semaphore);
 shared_memory_free(shared); // rimozione area memoria condivisa
 return 1;
}

