
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include "shared.h"
#include "semaphore.h"


#define CONSUMER_PROGRAM "./consumer"
#define BLOCK_SIZE 64
#define BUFFER_SIZE 4

#define TRUE 1
#define FALSE 0


int spawn(const char program[], char* arguments[])
{
 int pid;
 // clonazione del processo
 pid = fork ();
 if (pid < 0)
   return -1; // errore clonazione processo
 if (pid > 0)
   return 0; // codice di successo restituito al processo padre
 execv(program, arguments);
 abort(); // errore esecuzione programma
}


// struttura di definizione dell'area di memoria condivisa
struct SHARED
{
 unsigned char buffer[BLOCK_SIZE*BUFFER_SIZE]; // buffer blocchi dati
 unsigned int count; // numero di blocchi dati nel buffer
 int end; // flag di segnalazione della fine dei dati
 int space_semaphore; // semaforo per segnalazione spazio nel buffer
 int data_semaphore; // semaforo per segnalazione dati nel buffer
 int mutex_semaphore; // semaforo per accesso mutuamente esclusivo
};

FILE *input_file; // identificatore file di input
struct SHARED *shared; // puntatore area di memoria condivisa
int buffer_write_index; // indice per scrittura blocchi nel buffer


int main(int argc, char* argv[])
{
 int n;
 int key; // chiave per creazione area di memoria condivisa
 char buffer[BLOCK_SIZE]; // buffer temporaneo
 char* argument[2]; // argomento per invocazione programma

 if (argc != 3)
   {
    printf("Uso: %s input-file output-file\r\n",argv[0]);
    return 0;
   }
 // predisposizione argomenti invocazione programma
 argument[0] = (char*)malloc(strlen(CONSUMER_PROGRAM)+1);
 argument[1] = (char*)malloc(strlen(argv[2])+1);
 argument[2] = NULL;
 strcpy(argument[0], CONSUMER_PROGRAM); // nome del programma
 strcpy(argument[1], argv[2]); // argomento del programma
 // generazione chiave per allocazione area di memoria condivisa
 key = ftok(CONSUMER_PROGRAM, '@');
 if (key == -1)
   {
    printf("ftok error\r\n");
    return 0;
   }
 // allocazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, TRUE);
 if (shared == (struct SHARED*)(-1))
   {
    printf("shared_memory_alloc error\r\n");
    return 0;
   }
 // inizializzazione semafori
 shared->space_semaphore = semaphore_init(BUFFER_SIZE);
 if (shared->space_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    shared_memory_free(shared);
    return 0;
   }
 shared->data_semaphore = semaphore_init(0);
 if (shared->data_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    semaphore_remove(shared->space_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 shared->mutex_semaphore = semaphore_init(1);
 if (shared->mutex_semaphore == -1)
   {
    printf("semaphore_init error\r\n");
    semaphore_remove(shared->space_semaphore);
    semaphore_remove(shared->data_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 // apertura file di input in lettura
 input_file = fopen(argv[1], "rb");
 if (input_file == NULL)
   {
    printf("file open error\r\n");
    semaphore_remove(shared->space_semaphore);
    semaphore_remove(shared->data_semaphore);
    semaphore_remove(shared->mutex_semaphore);
    shared_memory_free(shared);
    return 0;
   }
 // inizializzazione variabili condivise e indice di scrittura
 buffer_write_index = 0;
 shared->count = 0;
 shared->end = FALSE;
 // esecuzione programma consumatore come processo separato
 if (spawn(CONSUMER_PROGRAM, argument) < 0)
   {
    printf("spawn error\r\n");
    semaphore_remove(shared->space_semaphore);
    semaphore_remove(shared->data_semaphore);
    semaphore_remove(shared->mutex_semaphore);
    shared_memory_free(shared);
    fclose(input_file);
    return 0;
   }
 // esecuzione ciclo processo produttore
 do
   {
    semaphore_wait(shared->space_semaphore);
    // lettura dati da file di input e copia nel buffer condiviso
    n = fread(buffer, 1, BLOCK_SIZE, input_file);
    memset( &(shared->buffer[buffer_write_index*BLOCK_SIZE]), 0, BLOCK_SIZE);
    memcpy( &(shared->buffer[buffer_write_index*BLOCK_SIZE]), buffer, n);
    // aggiornamento indice di scrittura nel file
    buffer_write_index = (buffer_write_index+1) % BUFFER_SIZE;
    semaphore_wait(shared->mutex_semaphore);
    shared->count++;
    if (feof(input_file))
      shared->end = TRUE;
    semaphore_signal(shared->mutex_semaphore);
    semaphore_signal(shared->data_semaphore);
   } while (!feof(input_file));
 // segnalazione per processo consumatore in attesa
 semaphore_signal(shared->data_semaphore);
 fclose(input_file); // chiusura file di input
 shared_memory_free(shared); // rimozione area memoria condivisa
 return 1;
}
