
#include <termios.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/ioctl.h>

#include "COM.h"


#define READ 0
#define WRITE 1
#define READ_WRITE 2
#define NO_HANDSHAKE 0
#define HW_HANDSHAKE 1


struct termios saveTioFlags; // struttura per salvataggio impostazioni correnti


int COM_open (unsigned char port, unsigned char mode, unsigned short speed, char parity, unsigned char bits, unsigned char stop, unsigned char flow)
{
 int PortId;
 struct termios SerialFlags;
 char dev[] = "/dev/ttyS?"; // dipendente dalla distribuzione Linux

 if (port > 9)
   return -1;
 dev[9] = (port+'0'); // trasformazione cifra numerica in carattere
 
 // apertura file speciale
 switch (mode)
       {
        case READ: 				PortId = open(dev, O_RDONLY);
													break;
				case WRITE: 			PortId = open(dev, O_WRONLY);
													break;
				case READ_WRITE: 	PortId = open(dev, O_RDWR);
													break;
				default: 					return -1;
       }

 if (PortId < 0)
   {
    close(PortId);
    return -1;
   }

 // acquisizione e salvataggio impostazioni correnti per successivo ripristino
 if (tcgetattr(PortId, &saveTioFlags) < 0 || tcgetattr(PortId, &SerialFlags) < 0)
   {
    close(PortId);
    return -1; 
   }
 
 // impostazione modalit "raw" non bloccante in lettura
 SerialFlags.c_lflag &= ~(ICANON | ECHO | ECHOE | ISIG);
 SerialFlags.c_lflag |= NOFLSH;
 SerialFlags.c_iflag &= ~(IXON | IXOFF | IXANY);
 SerialFlags.c_oflag &= ~OPOST;
 SerialFlags.c_cflag |= (CREAD|CLOCAL);
 SerialFlags.c_cc[VTIME] = 0;
 SerialFlags.c_cc[VMIN] = 1;
 
 // impostazione baudrate
 switch (speed)
       {
				case 4800: 	cfsetispeed(&SerialFlags, B4800);
										cfsetospeed(&SerialFlags, B4800);
										break;
				case 9600: 	cfsetispeed(&SerialFlags, B9600);
										cfsetospeed(&SerialFlags, B9600);
										break;
				case 19200: cfsetispeed(&SerialFlags, B19200);
										cfsetospeed(&SerialFlags, B19200);
										break;
				case 38400: cfsetispeed(&SerialFlags, B38400);
										cfsetospeed(&SerialFlags, B38400);
										break;
				case 57600: cfsetispeed(&SerialFlags, B57600);
										cfsetospeed(&SerialFlags, B57600);
										break;
				default: 		close(PortId);
										return -2;
       }

 // impostazione parita'
 SerialFlags.c_cflag &= ~(PARENB | PARODD);
 switch (parity)
       {
				case 'N':
				case 'n': break;
				case 'O':
				case 'o': SerialFlags.c_cflag |= (PARENB|PARODD);
									SerialFlags.c_iflag |= (INPCK);
									break;
				case 'E':
				case 'e': SerialFlags.c_cflag |= PARENB;
									SerialFlags.c_iflag |= (INPCK);
									break;
				default: 	close(PortId);
									return -3;
        }

 // impostazione numero bit carattere
 SerialFlags.c_cflag &= ~CSIZE;
 switch (bits)
       {
				case 7:		SerialFlags.c_cflag |= CS7;
									break;
				case 8: 	SerialFlags.c_cflag |= CS8;
									break;
				default:	close(PortId);
									return -4;
        }

 // impostazione numero stop bit
 SerialFlags.c_cflag &= ~CSTOPB;
 switch (stop)
       {
				case 1: 	break;
				case 2: 	SerialFlags.c_cflag |= CSTOPB;
									break;
				default:	close(PortId);
									return -5;
       }
       
 // impostazione CTS/RTS handshake
 SerialFlags.c_cflag &= ~CRTSCTS;
 switch (flow)
       {
				case NO_HANDSHAKE: 	break;
				case HW_HANDSHAKE: 	SerialFlags.c_cflag |= CRTSCTS;
														break;
				default: 						close(PortId);
														return -6;
       }

 // configurazione porta seriale con impostazioni effettuate
 if (tcsetattr(PortId, TCSANOW, &SerialFlags) < 0)
   {
    close(PortId);
    return -6;
   }

 return PortId;
}

int COM_write (int com, char buffer[], int N)
{
 return write(com, buffer, N);
}

int COM_read (int com, char *buffer, int N)
{
 return read(com, buffer, N);
}

void COM_close(int com)
{
 tcsetattr(com, TCSANOW, &saveTioFlags);
 close(com);
 return;
}
