
#include <string.h>
#include <stdlib.h>
#include <math.h>
#include "COM.h"
#include "GPS.h"

static int com_id = -1; // identificatore associato alla porta seriale aperta
static char linea[128]; // stringa per la ricostruzione del messaggio NMEA generato dal dispositivo GPS
static unsigned int indice; // indice della prima posizione disponibile nella stringa di ricostruzione del messaggio NMEA


// estrazione dei dati da una stringa NMEA di tipo $GPRMC e costruzione della struttura GPS_DATA
static struct GPS_DATA extractData(char nmea_string[], int* valid)
{
  char substring[16];
  unsigned int degree, hour, minute, second, day, month, year;
  double tmp;
  int pos, date_pos;
  struct GPS_DATA data;

  if (nmea_string[VALIDITY_POS] != 'A') // dati invalidi
    {
	 *valid = 0;
	 return data;
    }

  pos = SPEED_POS;
  while (nmea_string[pos] != ',')
	   pos++;
  pos++;
  while (nmea_string[pos] != ',')
	   pos++;
  pos++;
  date_pos = pos;
  
  // estrazione orario
  strncpy(substring,&nmea_string[TIME_POS],2);
  substring[2] = '\0';
  hour = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[TIME_POS+2],2);
  substring[2] = '\0';
  minute = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[TIME_POS+4],2);
  substring[2] = '\0';
  second = (unsigned int)atoi(substring);
  data.hour = hour;
  data.minute = minute;
  data. second = second;
  
  // estrazione data
  strncpy(substring,&nmea_string[date_pos],2);
  substring[2] = '\0';
  day = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[date_pos+2],2);
  substring[2] = '\0';
  month = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[date_pos+4],2);
  substring[2] = '\0';
  year = 2000+(unsigned int)atoi(substring);
  data.day = day;
  data.month = month;
  data.year = year;
  
  // estrazione latitudine
  strncpy(substring,&nmea_string[LAT_POS],2);
  substring[2] = '\0';
  degree = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[LAT_POS+2],7);
  substring[7] = '\0';
  tmp = atof(substring);
  minute = (unsigned int)floor(tmp);
  tmp = (tmp - floor(tmp))*60;
  second = (unsigned int)floor(tmp+0.5);
  data.lat_sign = nmea_string[LAT_SIGN_POS];
  data.lat_degree = degree;
  data.lat_minute = minute;
  data.lat_second = second;
  
  // estrazione longitudine
  strncpy(substring,&nmea_string[LONG_POS],3);
  substring[3] = '\0';
  degree = (unsigned int)atoi(substring);
  strncpy(substring,&nmea_string[LONG_POS+3],7);
  substring[7] = '\0';
  tmp = atof(substring);
  minute = (unsigned int)floor(tmp);
  tmp = (tmp - floor(tmp))*60;
  second = (unsigned int)floor(tmp+0.5);
  data.long_sign = nmea_string[LONG_SIGN_POS];
  data.long_degree = degree;
  data.long_minute = minute;
  data.long_second = second;
  
  *valid = 1;
  return data;
}
 
 

void GPS_open(unsigned char serial_port)
{
  com_id = COM_open(serial_port, READ, 4800, 'N', 8, 1, NO_HANDSHAKE);
  indice = 0;
}

void GPS_close(void)
{
  COM_close(com_id);
  com_id = -1;
}

int GPS_get(struct GPS_DATA* data)
{
  int n,i;
  char buffer[1024];
  int valid;

  if (com_id != -1)
    {
     n = COM_read(com_id, buffer, sizeof(buffer)); // ricezione caratteri
	 if (n > 0)
       {
        for (i=0; i<n; i++) // ricerca carattere fine linea
	       {
		    if (buffer[i] == '$')
		      {
		   	   linea[0] = buffer[i];
			   indice = 1;
			  }
		   else
		      {
			   if (buffer[i] == '\n')
			     {
				  linea[indice] = '\0';
                  //printf("<%s>\r\n", linea);
				  if (strncmp(linea,"$GPRMC",6)==0)
				    {
					 *data = extractData(linea, &valid); // estrazione dati da stringa GPS
					 if (valid)
					   return 1;
 				    }
				  indice = 0;
				 }
			   else
				   {
				    linea[indice] = buffer[i];
					indice++;
				   }
			  } // else
		   } // for
	   } // if
	 return 0;
	} // if
  return -1;
}
