#include <unistd.h>
#include <sys/ipc.h>
#include <stdio.h>
#include <stdlib.h>

#include "shared.h"
#include "semaphore.h"


#define KEY_FILE	"key_file"

struct GPS_DATA
{
 // timestamp
 unsigned short hour;
 unsigned short minute;
 unsigned short second;
 unsigned short day;
 unsigned short month;
 unsigned short year;
 // posizione geografica
 char lat_sign; // N/S
 unsigned short lat_degree;
 unsigned short lat_minute;
 unsigned short lat_second;
 char long_sign; // E/W
 unsigned short long_degree;
 unsigned short long_minute;
 unsigned short long_second;
};

// struttura di definizione dell’area di memoria condivisa
struct SHARED
{
 struct GPS_DATA last_position;
 int valid;
 int mutex_semaphore;
 int reader_semaphore;
 int reader_counter;
};




int main(int argc, char* argv[])
{
 int valid, key;
 struct GPS_DATA gps_data;
 struct SHARED *shared;
 
 // generazione chiave per associazione area di memoria condivisa
 key = ftok(KEY_FILE, '@');
 if (key == -1)
   {
    printf("Errore generazione chiave\r\n");
    return -1;
   }
 // associazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, 0);
 if (shared == (struct SHARED*)(-1))
   {
    printf("Errore associazione area di memoria condivisa\r\n");
    return -1;
   }
 
 while (1) // ripetizione senza fine
      {
       usleep((rand()%5)*1000000);
       // lettura dei dati nell’area di memoria condivisa in modalita’ mutuamente esclusiva
       semaphore_wait(shared->reader_semaphore);
       shared->reader_counter++;
       if (shared->reader_counter == 1)
         semaphore_wait(shared->mutex_semaphore);
       semaphore_signal(shared->reader_semaphore);
       gps_data = shared->last_position;
       valid = shared->valid;
       semaphore_wait(shared->reader_semaphore);
       shared->reader_counter--;
       if (shared->reader_counter == 0)
         semaphore_signal(shared->mutex_semaphore);
       semaphore_signal(shared->reader_semaphore);
       if (valid)
	     {
	      printf("%02d:%02d:%02d %02d/%02d/%04d %02d°%02d'%02d\"%c %03d°%02d'%02d\"%c\r\n",
	             gps_data.hour, gps_data.minute, gps_data.second, gps_data.day, gps_data.month, gps_data.year,
		         gps_data.lat_degree, gps_data.lat_minute, gps_data.lat_second, gps_data.lat_sign,
		         gps_data.long_degree, gps_data.long_minute, gps_data.long_second, gps_data.long_sign);
	     }
       else
	       printf("Dati GPS non validi\r\n");
      }

  shared_memory_free(shared);
  return 0;
}
