#include <unistd.h>
#include <sys/ipc.h>
#include <stdio.h>
#include <stdlib.h>

#include "GPS.h"
#include "shared.h"
#include "semaphore.h"


#define KEY_FILE	"key_file"


// struttura di definizione dell’area di memoria condivisa
struct SHARED
{
 struct GPS_DATA last_position;
 int valid;
 int mutex_semaphore;
 int reader_semaphore;
 int reader_counter;
};




int main(int argc, char* argv[])
{
 int serial_port, key;
 struct GPS_DATA gps_data;
 struct SHARED *shared;
 
 if (argc != 2) // verifica presenza argomento
   {
    printf("Uso: %s GPS-port\r\n", argv[0]);
    return -1;
   }
 
 serial_port = atoi(argv[1]);

 if ((serial_port<0) || (serial_port>9)) // verifica valore argomento
   {
    printf("Errore: numero di porta errato\r\n");
    return -1;
   }

 // connessione dispositivo GPS
 GPS_open(serial_port);

 // generazione chiave per allocazione area di memoria condivisa
 key = ftok(KEY_FILE, '@');
 if (key == -1)
   {
    printf("Errore generazione chiave\r\n");
	GPS_close();
    return -1;
   }
 printf("KEY generata\r\n");
 // allocazione area memoria condivisa
 shared = (struct SHARED*)shared_memory_alloc_key(sizeof(struct SHARED), key, 1);
 if (shared == (struct SHARED*)(-1))
   {
    printf("Errore allocazione area di memoria condivisa\r\n");
	GPS_close();
    return -1;
   }
 printf("SHARED allocata\r\n");
 // inizializzazione semafori
 shared->mutex_semaphore = semaphore_init(1);
 if (shared->mutex_semaphore == -1)
   {
    printf("Errore inizializzazione semaforo MUTEX\r\n");
    shared_memory_free(shared);
	GPS_close();
    return -1;
   }
 shared->reader_semaphore = semaphore_init(1);
 if (shared->reader_semaphore == -1)
   {
    printf("Errore inizializzazione semaforo READER\r\n");
    shared_memory_free(shared);
    semaphore_remove(shared->mutex_semaphore);
    GPS_close();
    return -1;
   }
 printf("READER inizializzato\r\n");
 shared->valid = 0; // segnalazione dati non validi
 shared->reader_counter = 0;
 while (1) // ripetizione senza fine 
      if (GPS_get(&gps_data)) // dati GPS validamente aggiornati
	{
         // scrittura dei dati nell’area di memoria condivisa in modalita’ mutuamente esclusiva
         semaphore_wait(shared->mutex_semaphore);
	     shared->last_position = gps_data;
	     shared->valid = 1; // segnalazione dati validi
	     semaphore_signal(shared->mutex_semaphore);
        }
 
 semaphore_remove(shared->mutex_semaphore);
 semaphore_remove(shared->reader_semaphore);
 shared_memory_free(shared);
 GPS_close();
 return 0;
}
